<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Database.inc.php                                               04/23/2003
 * This script contains a set of functions that may be useful when dealing with
 * Virtual Advanced database groups, databases, and messages.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2003 Steve Winn. All Rights Reserved.
 *
 * The source code contains a fair amount of comments, which may aid anyone
 * attempting to follow the program or to modify it.
 *
 * For licensing and copyright information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('va-functions.inc.php');       // A VADV-PHP script.


/**
* Va-Database.inc.php :: FindTopicInfo()
*
* This function will retrieve the variables for the selected database topic using
* the passed file string. This function is identical to the GetTopicInfo function
* except you MUST pass the file string to it. This is useful if you are going to
* search the topics multiple times.
*
* @param string $type
*/
function FindTopicInfo($topicNo, $fileString, $type='') {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) trigger_error(VA_Error(1));
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ($topicNo < 1) $topicNo = 1;

    global $VA_SYSPATHS;                    // Include global variable.

    $type  = strtoupper($type);             // Change the type to uppercase.
    $topic = chr($topicNo + 64);            // Change the topic number into the topic letter.

    $topicInfo = FALSE;
    $offset    = 128;                       // Set the file offset.
    $position  = 2;                         // Set the starting position.
    $dbType    = 'tmp';
    $string    = 'tmp';

    for ($counter = 1; $dbType != ''; $counter++) {
        $dbType = substr($fileString, $position - 2, 1);
        if ($type != '') {
            if ($type == $dbType) {
                $string = rtrim(substr($fileString, $position - 1, 1));
                $string = strtoupper($string);
                if ($string == $topic) {
                    $offset = 128 * ($counter - 1);
                    $topicInfo['topicno'] = $topicNo;
                    $topicInfo['topic']   = $topic;
                    $topicInfo['dbtype']  = GetString($fileString, 1 + $offset, 1);
                    $topicInfo['topic']   = GetString($fileString, 2 + $offset, 1);
                    $topicInfo['desc']    = GetString($fileString, 3 + $offset, 65);
                    $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                    $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                    $topicInfo['dbno']    = GetDBCount($topicNo);
                    $dbType = '';
                }
            }
        } else {
            $string = rtrim(substr($fileString, $position - 1, 1));
            $string = strtoupper($string);
            if ($string == $topic) {
                $offset = 128 * ($counter - 1);
                $topicInfo['topicno'] = $topicNo;
                $topicInfo['topic']   = $topic;
                $topicInfo['dbtype']  = GetString($fileString, 1 + $offset, 1);
                $topicInfo['topic']   = GetString($fileString, 2 + $offset, 1);
                $topicInfo['desc']    = GetString($fileString, 3 + $offset, 65);
                $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                $topicInfo['dbno']    = GetDBCount($topicNo);
                $dbType = '';
            }
        }
        $position = $position + $offset;
    }

    return $topicInfo;                      // Return the topic information.
}


/**
* Va-Database.inc.php :: GetTopicInfo()
*
* This function will retrieve the variables for the selected database topic.
*
* @param string $type
*/
function GetTopicInfo($topicNo, $type='') {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) trigger_error(VA_Error(1));
    if ($topicNo < 1) $topicNo = 1;

    global $VA_SYSPATHS;                    // Include global variable.
    
    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '/dbgroup.cfg';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    $type  = strtoupper($type);             // Change the type to uppercase.
    $topic = chr($topicNo + 64);            // Change the topic number into the topic letter.

    $topicInfo = FALSE;
    $offset    = 128;                       // Set the file offset.
    $position  = 2;                         // Set the starting position.
    $dbType    = 'tmp';
    $string    = 'tmp';

    for ($counter = 1; $dbType != ''; $counter++) {
        $dbType = substr($fileString, $position - 2, 1);
        if ($type != '') {
            if ($type == $dbType) {
                $string = rtrim(substr($fileString, $position - 1, 1));
                $string = strtoupper($string);
                if ($string == $topic) {
                    $offset = 128 * ($counter - 1);
                    $topicInfo['topicno'] = $topicNo;
                    $topicInfo['topic']   = $topic;
                    $topicInfo['dbtype']  = GetString($fileString, 1 + $offset, 1);
                    $topicInfo['topic']   = GetString($fileString, 2 + $offset, 1);
                    $topicInfo['desc']    = GetString($fileString, 3 + $offset, 65);
                    $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                    $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                    $topicInfo['dbno']    = GetDBCount($topicNo);
                    $dbType = '';
                }
            }
        } else {
            $string = rtrim(substr($fileString, $position - 1, 1));
            $string = strtoupper($string);
            if ($string == $topic) {
                $offset = 128 * ($counter - 1);
                $topicInfo['topicno'] = $topicNo;
                $topicInfo['topic']   = $topic;
                $topicInfo['dbtype']  = GetString($fileString, 1 + $offset, 1);
                $topicInfo['topic']   = GetString($fileString, 2 + $offset, 1);
                $topicInfo['desc']    = GetString($fileString, 3 + $offset, 65);
                $topicInfo['marker1'] = GetLongInt($fileString, 70 + $offset);
                $topicInfo['marker2'] = GetLongInt($fileString, 74 + $offset);
                $topicInfo['dbno']    = GetDBCount($topicNo);
                $dbType = '';
            }
        }
        $position = $position + $offset;
    }

    return $topicInfo;                      // Return the topic information.
}


/**
* Va-Database.inc.php :: GetTopics()
*
* This function will retrieve the topic numbers and store them into an array.
*
* @param string $type
*/
function GetTopics($type='') {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '/dbgroup.cfg';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    $type = strtoupper($type);              // Change the topic type into uppercase.

    $offset     = 128;                      // Set the file offset.
    $position   = 2;                        // Set the starting position.
    $string     = 'tmp';
    $topics     = array('');

    for ($counter = 1; $string != ''; $counter++) {
        $string = substr($fileString, $position - 2, 1);
        if ($type != '') {
            if ($type == $string) {
                if ($string != '') {
                    $group = rtrim(substr($fileString, $position - 1, 1));
                    $group = strtoupper($group);
                    $group = ord($group) - 64;
                    if (GetDbCount($group) > 0) array_push($topics, $group);
                }
            }
        } else {
                if ($string != '') {
                    $group = rtrim(substr($fileString, $position - 1, 1));
                    $group = strtoupper($group);
                    $group = ord($group) - 64;
                    if (GetDbCount($group) > 0) array_push($topics, $group);
                }
        }
        $position = $position + $offset;
    }

    return $topics;                        // Return the topic numbers.
}


/**
* Va-Database.inc.php :: GetTopicCount()
*
* This function will determine the total number of topics and return that value.
*
* @param string $type
*/
function GetTopicCount($type = '') {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '/dbgroup.cfg';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    $type = strtoupper($type);              // Change the topic type into uppercase.

    $offset     = 128;                      // Set the file offset.
    $position   = 2;                        // Set the starting position.
    $string     = 'tmp';
    $topicNo    = 0;

    for ($counter = 1; $string != ''; $counter++) {
        $string = substr($fileString, $position - 2, 1);
        if ($type != '') {
            if ($type == $string) {
                if ($string != '') {
                    $group = rtrim(substr($fileString, $position - 1, 1));
                    $group = strtoupper($group);
                    $group = ord($group) - 64;
                    if (GetDbCount($group) > 0) $topicNo++;
                }
            }
        } else {
                if ($string != '') {
                    $group = rtrim(substr($fileString, $position - 1, 1));
                    $group = strtoupper($group);
                    $group = ord($group) - 64;
                    if (GetDbCount($group) > 0) $topicNo++;
                }
        }
        $position = $position + $offset;
    }

    return $topicNo;                        // Return the topic count.
}


/**
* Va-Database.inc.php :: FindDbInfo()
*
* This function will retrieve the variables for the selected database from the
* passed fileString. This function is identical to GetDbInfo except you MUST
* pass the fileString. This function is more useful if you are going to be
* searching the same database multiple times.
*
*/
function FindDbInfo($topicNo, $db, $fileString) {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) trigger_error(VA_Error(1));
    if ((!is_int($db))      && (!is_string($db)))      trigger_error(VA_Error(1));
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ($topicNo < 1) $topicNo = 1;
    if ($db < 1) $db = 1;

    $topic = chr($topicNo + 64);            // Convert the topic number into the topic letter.

    global $VA_SYSPATHS;                    // Include the global variable.

    $dbInfo   = array();
    $offset   = 131;
    $position = 75;
    $string   = 'tmp';
    $num      = 0;

    for ($counter = 1; $string != ''; $counter++) {
        $string   = rtrim(substr($fileString, $position - 1, 1));
        $string   = strtoupper($string);    // Convert the topic letter to uppercase.
        $position = $position + 131;
        if ($string == $topic) {
            $num++;
            if ($num == $db) {
                $offset = 131 * ($counter - 1);
                $dbInfo['topicno']    = $topicNo;
                $dbInfo['topic']      = $topic;
                $dbInfo['dbno']       = $db;
                $dbInfo['filename']   = GetString($fileString, 1 + $offset, 8);
                $dbInfo['filepath']   = GetString($fileString, 9 + $offset, 32);
                $dbInfo['desc']       = GetString($fileString, 41 + $offset, 32);
                $dbInfo['private']    = GetInteger($fileString, 73 + $offset);
                $dbInfo['dbgroup']    = GetString($fileString, 75 + $offset, 1);
                $dbInfo['maxmsg']     = GetInteger($fileString, 76 + $offset);
                $dbInfo['minreadsl']  = GetInteger($fileString, 78 + $offset);
                $dbInfo['minwritesl'] = GetInteger($fileString, 80 + $offset);
                $dbInfo['agelimit']   = GetInteger($fileString, 82 + $offset);
                $dbInfo['accflags']   = GetAccFlags(GetLongInt($fileString, 84 + $offset));
                $dbInfo['tagline']    = GetInteger($fileString, 88 + $offset);
                $dbInfo['dispname']   = GetInteger($fileString, 90 + $offset);
                $dbInfo['randtitle']  = GetInteger($fileString, 92 + $offset);
                $dbInfo['qwkconfno']  = GetInteger($fileString, 94 + $offset);
                $dbInfo['moderator']  = GetInteger($fileString, 96 + $offset);
                $dbInfo['validate']   = GetInteger($fileString, 98 + $offset);
                $dbInfo['freqable']   = GetInteger($fileString, 100 + $offset);
                $dbInfo['devicetype'] = GetInteger($fileString, 102 + $offset);
                $dbInfo['dbtype']     = GetString($fileString, 104 + $offset, 1);
                $dbInfo['msgno']      = GetMessageCount($dbInfo);
                if ($dbInfo['msgno'] == '') $dbInfo['msgno'] = 0;

                $string = '';
            }
        }
    }

    return $dbInfo;                         // Return the database information.
}


/**
* Va-Database.inc.php :: GetDbInfo()
*
* This function will retrieve the variables for the selected database.
*
*/
function GetDbInfo($topicNo, $db) {
    if ((!is_int($topicNo)) && (!is_string($topicNo))) trigger_error(VA_Error(1));
    if ((!is_int($db))      && (!is_string($db)))      trigger_error(VA_Error(1));
    if ($topicNo < 1) $topicNo = 1;
    if ($db < 1) $db = 1;

    $topic = chr($topicNo + 64);            // Convert the topic number into the topic letter.

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['main'] . '/database.cfg';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    $dbInfo   = array();
    $offset   = 131;
    $position = 75;
    $string   = 'tmp';
    $num      = 0;

    for ($counter = 1; $string != ''; $counter++) {
        $string   = rtrim(substr($fileString, $position - 1, 1));
        $string   = strtoupper($string);    // Convert the topic letter to uppercase.
        $position = $position + 131;
        if ($string == $topic) {
            $num++;
            if ($num == $db) {
                $offset = 131 * ($counter - 1);
                $dbInfo['topicno']    = $topicNo;
                $dbInfo['topic']      = $topic;
                $dbInfo['dbno']       = $db;
                $dbInfo['filename']   = GetString($fileString, 1 + $offset, 8);
                $dbInfo['filepath']   = GetString($fileString, 9 + $offset, 32);
                $dbInfo['desc']       = GetString($fileString, 41 + $offset, 32);
                $dbInfo['private']    = GetInteger($fileString, 73 + $offset);
                $dbInfo['dbgroup']    = GetString($fileString, 75 + $offset, 1);
                $dbInfo['maxmsg']     = GetInteger($fileString, 76 + $offset);
                $dbInfo['minreadsl']  = GetInteger($fileString, 78 + $offset);
                $dbInfo['minwritesl'] = GetInteger($fileString, 80 + $offset);
                $dbInfo['agelimit']   = GetInteger($fileString, 82 + $offset);
                $dbInfo['accflags']   = GetAccFlags(GetLongInt($fileString, 84 + $offset));
                $dbInfo['tagline']    = GetInteger($fileString, 88 + $offset);
                $dbInfo['dispname']   = GetInteger($fileString, 90 + $offset);
                $dbInfo['randtitle']  = GetInteger($fileString, 92 + $offset);
                $dbInfo['qwkconfno']  = GetInteger($fileString, 94 + $offset);
                $dbInfo['moderator']  = GetInteger($fileString, 96 + $offset);
                $dbInfo['validate']   = GetInteger($fileString, 98 + $offset);
                $dbInfo['freqable']   = GetInteger($fileString, 100 + $offset);
                $dbInfo['devicetype'] = GetInteger($fileString, 102 + $offset);
                $dbInfo['dbtype']     = GetString($fileString, 104 + $offset, 1);
                $dbInfo['msgno']      = GetMessageCount($dbInfo);
                if ($dbInfo['msgno'] == '') $dbInfo['msgno'] = 0;

                $string = '';
            }
        }
    }

    return $dbInfo;                         // Return the database information.
}


/**
* Va-Database.inc.php :: GetDbCount()
*
* This function will get the number of available databases in the selected
* topic. It will filter out those which the user does not have access to.
*
*/
function GetDbCount($topic) {
    if ((!is_int($topic)) && (!is_string($topic))) trigger_error(VA_Error(1));
    if ($topic < 0) trigger_error(VA_Error(2));

    $topic = chr($topic + 64);

    global $VA_SYSPATHS;                    // Include the global variable.

    $file = $VA_SYSPATHS['main'] . '/database.cfg';
    if (!is_file($file)) return FALSE;
    
    $fileString = ReadCfgFile($file);       // Read the file.

    $offset     = 131;                      // Set the file offset.
    $position   = 75;                       // Set the starting position.
    $string     = 'tmp';
    $dbNo       = 0;

    for ($counter = 1; $string != ''; $counter++) {
        $string = rtrim(substr($fileString, $position - 1, 1));
        $string = strtoupper($string);
        $position = $position + 131;
        if ($string == $topic) {
            $dbInfo = array();
            $offset = 131 * ($counter - 1);
            $dbInfo['minreadsl'] = GetInteger($fileString, 78 + $offset);
            $dbInfo['agelimit']  = GetInteger($fileString, 82 + $offset);
            $dbInfo['accflags']  = GetLongInt($fileString, 84 + $offset);
            if (DbAccess($dbInfo) != 0) {
                $dbNo++;
            }
        }
    }

    return $dbNo;                           // Return the database count.
}


/**
* Va-Database.inc.php :: GetMessagePointer()
*
* This function will return the specified user's message pointer for the
* current database.
*
*/
function GetMessagePointer($dbInfo, $userNo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($userNo))  && (!is_string($userNo))) trigger_error(VA_Error(1));
    if ($userNo < 1) return 32760;
    
    global $VA_SYSPATHS;                    // Include the global variable.
    
    $file = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'] . '.QSC';
    if (!is_file($file)) return 0;

    $fileString = ReadCfgFile($file);
    $offset = ($userNo - 1) * 4;
    $pointer = GetLongInt($fileString, 1 + $offset);
    
    return $pointer;                        // Return the message pointer.
}


/**
* Va-Database.inc.php :: WriteMessagePointer()
*
* This function will update the message pointer file for the specified database
* with either the supplied message number, or if no message number is passed,
* then it will increase the pointer by one.
*
*/
function WriteMessagePointer($dbInfo, $userNo, $msgNo=0) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($userNo))  && (!is_string($userNo))) trigger_error(VA_Error(1));
    if ($userNo < 1) return 32760;

    global $VA_SYSPATHS;                    // Include the global variable.

    $file = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'] . '.QSC';
    if (!is_file($file)) {
        $fileString = str_pad('', $userNo * 4, chr(0));
    } else {
        $fileString = ReadCfgFile($file);
    }
    $offset = ($userNo - 1) * 4;
    $pointer = GetLongInt($fileString, 1 + $offset);
    
    if ($msgNo != 0) {
        $newPointer = $msgNo;
    } else {
        $newPointer = $pointer + 1;
    }

    $fileString = PutLongInt($fileString, $newPointer, 1 + $offset);
    $status = WriteCfgFile($file, $fileString);
    if (!$status) return $status;

    return $pointer;                        // Return the message pointer.
}


/**
* Va-Database.inc.php :: GetTotalMessageCount()
*
* This function will return the total number of database entries.
*
*/
function GetTotalMessageCount() {
    global $VA_SYSPATHS;                    // Include the global variable.

    $file = $VA_SYSPATHS['data'] . '/dbseq.dat';
    if (!is_file($file)) return 0;

    $fileString = ReadCfgFile($file);       // Read the file.
    return GetLongInt($fileString, 1);      // Return the total message count.
}


/**
* Va-Database.inc.php :: WriteTotalMessageCount()
*
* This function will add one to the total message count and return that integer.
*
*/
function WriteTotalMessageCount() {
    global $VA_SYSPATHS;                    // Include the global variable.

    $file = $VA_SYSPATHS['data'] . '/dbseq.dat';
    if (!is_file($file)) {
        $fileString = '0';
        $fileString = PutLongInt($fileString, 1, 1);
        $status = WriteCfgFile($file, $fileString);
        if (!$status) return $status;
        return 1;
    }

    $fileString = ReadCfgFile($file);       // Read the file.
    $msgcnt = GetLongInt($fileString, 1);   // Get the total message count.

    $fileString = PutLongInt($fileString, $msgcnt + 1, 1);
    $status = WriteCfgFile($file, $fileString);
    if (!$status) return $status;
    
    return $msgcnt;                         // Return the original value.
}


/**
* Va-Database.inc.php :: FindMessageInfo()
*
* This function will retrieve the variables for the specified message using the
* passed fileString. This function is identical to the GetMessageInfo function
* except you must pass the fileString. This is useful if you are going to be
* searching the same message database multiple times.
*
*/
function FindMessageInfo($dbInfo, $msgNo, $fileString) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($msgNo))   && (!is_string($msgNo))) trigger_error(VA_Error(1));
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    if ($msgNo < 1) $msgno = 1;
    if ($msgNo > $dbInfo['msgno']) return FALSE;

    global $VA_SYSPATHS;                    // Include global variable.

    $msgInfo = array();
    $offset  = 582 * ($msgNo - 1);
    $msgInfo['topicno']     = $dbInfo['topicno'];
    $msgInfo['topic']       = $dbInfo['topic'];
    $msgInfo['dbno']        = $dbInfo['dbno'];
    $msgInfo['fromuserno']  = GetLongInt($fileString, 1 + $offset);
    $msgInfo['touserno']    = GetLongInt($fileString, 5 + $offset);
    $msgInfo['fromhandle']  = GetString($fileString, 9 + $offset, 40);
    if ($msgInfo['fromhandle'] == '') $msgInfo['fromhandle'] = 'Unknown';
    $msgInfo['tohandle']    = GetString($fileString, 49 + $offset, 40);
    if ($msgInfo['tohandle'] == '') $msgInfo['tohandle'] = 'All';
    $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
    $msgInfo['tonetnode']   = GetString($fileString, 169 + $offset, 80);
    $msgInfo['fromnetid']   = GetInteger($fileString, 249 + $offset);
    $msgInfo['tonetid']     = GetInteger($fileString, 251 + $offset);
    $msgInfo['timestamp']   = GetLongInt($fileString, 253 + $offset) - (date("O") / 100 * 60 * 60);
    $msgInfo['daysold']     = intval((time() - $msgInfo['timestamp']) / (60 * 60 * 24));
    $msgInfo['sysmsgno']    = GetLongInt($fileString, 257 + $offset);
    $msgInfo['subject']     = GetString($fileString, 269 + $offset, 64);
    $msgInfo['deleted']     = GetInteger($fileString, 333 + $offset);
    $msgInfo['creation']    = GetString($fileString, 335 + $offset, 38);
    $msgInfo['date']        = GetString($fileString, 335 + $offset, 17);
    $msgInfo['time']        = GetString($fileString, 352 + $offset, 9);
    $msgInfo['timezone']    = GetString($fileString, 361 + $offset, 12);
    $msgInfo['offline']     = GetInteger($fileString, 375 + $offset);
    $msgInfo['attfilesize'] = GetLongInt($fileString, 379 + $offset);
    $msgInfo['attfilename'] = GetString($fileString, 383 + $offset, 12);
    $msgInfo['attfilepath'] = GetString($fileString,395 + $offset, 64);
    $msgInfo['downloadno']  = GetInteger($fileString, 463 + $offset);
    $msgInfo['filename']    = GetString($fileString, 471 + $offset, 8);
    $msgInfo['msgpos']      = GetLongInt($fileString, 479 + $offset);
    $msgInfo['msglength']   = GetLongInt($fileString, 483 + $offset);
    $msgInfo['originnet']   = GetLongInt($fileString, 487 + $offset);
    $msgInfo['originnode']  = GetLongInt($fileString, 491 + $offset);

    return $msgInfo;                        // Return the message information.
}


/**
* Va-Database.inc.php :: GetMessageInfo()
*
* This function will retrieve the variables for the specified message.
*
*/
function GetMessageInfo($dbInfo, $msgNo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($msgNo))   && (!is_string($msgNo))) trigger_error(VA_Error(1));
    if ($msgNo < 1) $msgno = 1;
    if ($msgNo > $dbInfo['msgno']) return FALSE;

    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];

    $datFile = $file . '.DAT';              // Set the database filename.
    if (!is_file($datFile)) return FALSE;

    $fileString = ReadCfgFile($datFile);    // Read the file.

    $msgInfo = array();
    $offset  = 582 * ($msgNo - 1);
    $msgInfo['topicno']     = $dbInfo['topicno'];
    $msgInfo['topic']       = $dbInfo['topic'];
    $msgInfo['dbno']        = $dbInfo['dbno'];
    $msgInfo['fromuserno']  = GetLongInt($fileString, 1 + $offset);
    $msgInfo['touserno']    = GetLongInt($fileString, 5 + $offset);
    $msgInfo['fromhandle']  = GetString($fileString, 9 + $offset, 40);
    if ($msgInfo['fromhandle'] == '') $msgInfo['fromhandle'] = 'Unknown';
    $msgInfo['tohandle']    = GetString($fileString, 49 + $offset, 40);
    if ($msgInfo['tohandle'] == '') $msgInfo['tohandle'] = 'All';
    $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
    $msgInfo['tonetnode']   = GetString($fileString, 169 + $offset, 80);
    $msgInfo['fromnetid']   = GetInteger($fileString, 249 + $offset);
    $msgInfo['tonetid']     = GetInteger($fileString, 251 + $offset);
    $msgInfo['timestamp']   = GetLongInt($fileString, 253 + $offset) - (date("O") / 100 * 60 * 60);
    $msgInfo['daysold']     = intval((time() - $msgInfo['timestamp']) / (60 * 60 * 24));
    $msgInfo['sysmsgno']    = GetLongInt($fileString, 257 + $offset);
    $msgInfo['subject']     = GetString($fileString, 269 + $offset, 64);
    $msgInfo['deleted']     = GetInteger($fileString, 333 + $offset);
    $msgInfo['creation']    = GetString($fileString, 335 + $offset, 38);
    $msgInfo['date']        = GetString($fileString, 335 + $offset, 17);
    $msgInfo['time']        = GetString($fileString, 352 + $offset, 9);
    $msgInfo['timezone']    = GetString($fileString, 361 + $offset, 12);
    $msgInfo['offline']     = GetInteger($fileString, 375 + $offset);
    $msgInfo['attfilesize'] = GetLongInt($fileString, 379 + $offset);
    $msgInfo['attfilename'] = GetString($fileString, 383 + $offset, 12);
    $msgInfo['attfilepath'] = GetString($fileString,395 + $offset, 64);
    $msgInfo['downloadno']  = GetInteger($fileString, 463 + $offset);
    $msgInfo['filename']    = GetString($fileString, 471 + $offset, 8);
    $msgInfo['msgpos']      = GetLongInt($fileString, 479 + $offset);
    $msgInfo['msglength']   = GetLongInt($fileString, 483 + $offset);
    $msgInfo['originnet']   = GetLongInt($fileString, 487 + $offset);
    $msgInfo['originnode']  = GetLongInt($fileString, 491 + $offset);

    return $msgInfo;                        // Return the message information.
}


/**
* Va-Database.inc.php :: WriteMessageInfo()
*
* This function will allow you to write new message information into the
* corresponding .DAT file. You cannot change the creation date.
*
*/
function WriteMessageInfo($dbInfo, $msgNo, $configVariable) {
    if (!is_array($dbInfo))                          trigger_error(VA_Error(1));
    if ((!is_int($msgNo))   && (!is_string($msgNo))) trigger_error(VA_Error(1));
    if (!is_array($configVariable))                  trigger_error(VA_Error(1));
    if ($msgNo < 1) return FALSE;
    if ($msgNo > $dbInfo['msgno']) return FALSE;


    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $datFile = $file . '.DAT';              // Set the database filename.
    if (!is_file($datFile)) return FALSE;

    $fileString = ReadCfgFile($datFile);    // Read the file.

    $offset = 582 * ($msgNo - 1);
    if (isset($configVariable['fromuserno']))  $fileString = PutLongInt($fileString, $configVariable['fromuserno'], 1 + $offset);
    if (isset($configVariable['touserno']))    $fileString = PutLongInt($fileString, $configVariable['touserno'], 5 + $offset);
    if (isset($configVariable['fromhandle']))  $fileString = PutString($fileString, $configVariable['fromhandle'], 9 + $offset, 40);
    if (isset($configVariable['tohandle']))    $fileString = PutString($fileString, $configVariable['tohandle'], 49 + $offset, 40);
    if (isset($configVariable['fromnetnode'])) $fileString = PutString($fileString, $configVariable['fromnetnode'], 89 + $offset, 80);
    if (isset($configVariable['tonetnode']))   $fileString = PutString($fileString, $configVariable['tonetnode'], 169 + $offset, 80);
    if (isset($configVariable['fromnetid']))   $fileString = PutInteger($fileString, $configVariable['fromnetid'], 249 + $offset);
    if (isset($configVariable['tonetid']))     $fileString = PutInteger($fileString, $configVariable['tonetid'], 251 + $offset);
    if (isset($configVariable['sysmsgno']))    $fileString = PutLongInt($fileString, $configVariable['sysmsgno'], 257 + $offset);
    if (isset($configVariable['subject']))     $fileString = PutString($fileString, $configVariable['subject'], 269 + $offset, 64);
    if (isset($configVariable['deleted']))     $fileString = PutInteger($fileString, $configVariable['deleted'], 333 + $offset);
    if (isset($configVariable['offline']))     $fileString = PutInteger($fileString, $configVariable['offline'], 375 + $offset);
    if (isset($configVariable['attfilesize'])) $fileString = PutLongInt($fileString, $configVariable['attfilesize'], 379 + $offset);
    if (isset($configVariable['attfilename'])) $fileString = PutString($fileString, $configVariable['attfilename'], 383 + $offset, 12);
    if (isset($configVariable['attfilepath'])) $fileString = PutString($fileString, $configVariable['attfilepath'], 395 + $offset, 64);
    if (isset($configVariable['downloadno']))  $fileString = PutInteger($fileString, $configVariable['downloadno'], 463 + $offset);
    if (isset($configVariable['filename']))    $fileString = PutString($fileString, $configVariable['filename'], 471 + $offset, 8);
    if (isset($configVariable['msgpos']))      $fileString = PutLongInt($fileString, $configVariable['msgpos'], 479 + $offset);
    if (isset($configVariable['msglength']))   $fileString = PutLongInt($fileString, $configVariable['msglength'], 483 + $offset);

    $status = WriteCfgFile($datFile, $fileString);

    return $status;                         // Return the result status.

}


/**
* Va-Database.inc.php :: GetMessage()
*
* This function will retrieve the message for the selected information.
*
*/
function GetMessage($dbInfo, $msgNo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($msgNo))   && (!is_string($msgNo))) trigger_error(VA_Error(1));
    if ($msgNo < 1) $msgNo = 1;
    if ($msgNo > $dbInfo['msgno']) return FALSE;

    global $VA_SYSPATHS;                    // Include the global variable.
    
    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];

    $datFile = $file . '.DAT';
    if (!is_file($datFile)) return FALSE;

    $binFile = $file . '.BIN';
    if (!is_file($binFile)) return FALSE;

    $fileString = ReadCfgFile($datFile);    // Read the file.

    $offset    = 582 * ($msgNo - 1);                        // Set the file offset.
    $msgPos    = GetLongInt($fileString, 479 + $offset) - 1;// Determine the message position.
    $msgLength = GetLongInt($fileString, 483 + $offset);    // Determine the message length.

    $fp = fopen($binFile, 'r');                             // Open the file.
    fseek($fp, $msgPos);
    $message = fread($fp, $msgLength);                      // Get the message.
    fclose($fp);                                            // Close the file.
    
    $message = rtrim(StripRouting(str_replace('&amp;', '&', htmlspecialchars($message))));

    return $message;                        // Return the message.
}


/**
* Va-Database.inc.php :: WriteMessage()
*
* This function will write the message header information and the message text
* to the proper files based on the passed database information. This function
* is to write only new message area messages. For emails or files, use the
* appropriate function. If this is a reply to a message, set $reply to the
* message number of the original message.
*
*/
function WriteMessage($dbInfo, $fromUserNo, $msgSubject, $msgBody, $reply=0) {
    if (!is_array($dbInfo))      trigger_error(VA_Error(1));
    if ((!is_int($fromUserNo))   && (!is_string($fromUserNo))) trigger_error(VA_Error(1));
    if (!is_string($msgSubject)) trigger_error(VA_Error(1));
    if (!is_string($msgBody))    trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                    // Include global variable.

    $fromUserNo = trim($fromUserNo);
    if ($fromUserNo < 1) return FALSE;
    $msgSubject = rtrim(strip_tags($msgSubject));
    if (strlen($msgSubject) > 64) $msgSubject = substr($msgSubject, 0, 64);
    $msgBody = rtrim(wordwrap(strip_tags($msgBody), 80, "\r\n", 1));
    if ($dbInfo['tagline'] != 0) {                                              // If tagline is set...
        $file = $VA_SYSPATHS['text'] . '/tagline.' . $dbInfo['tagline'];        // Look for the file...
        if (is_file($file)) $msgBody = $msgBody . "\r\n" . ReadCfgFile($file);  // Append the file if found.
    }

    include_once('va-userfile.inc.php');
    include_once('va-main.inc.php');
    include_once('va-networks.inc.php');
    include_once('va-stats.inc.php');
    global $VA_MAIN;                        // Include global variable in case it is already defined.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $binFile = $file . '.BIN';              // Set the bin filename.
    $datFile = $file . '.DAT';              // Set the database filename.

    // Get the 'from' user information and update it.
    $fileString = ReadUserfile();
    $fromUser = GetUser($fromUserNo);
    $fromUser['totalposts']++;
    $fromUser['credits'] = $fromUser['credits'] + $VA_MAIN['credperpost'];  // Award credits for posting.
    $fileString = PutUser($fileString, $fromUserNo, $fromUser);
    $status = WriteUserfile($fileString);

    // Update the BBS stats.
    $fileString = ReadStats();
    $stats = GetStats();
    $stats['poststotal']++;
    $stats['poststoday']++;
    $fileString = PutStats($fileString, $stats);
    $status = WriteStats($fileString);

    if (!is_file($binFile)) {
        $msgpos = 1;
    } else {
        $msgpos = filesize($binFile) + 1;       // Get the message starting position.
    }
    $msglen = strlen($msgBody) + 2;             // Get the message length.

    // Update the .DAT file (and update dbseq.dat).
    // Add spacing of 582 bytes of null chars.
    $newString = str_pad('', 582, chr(0));  // Create the new blank string.
    $networks = GetNetworkInfo();           // Get the local network info.

    // Check for special database settings.
    if ($dbInfo['dispname'] == 1) {
        $fromHandle = $fromUser['realname'];
    } else if ($dbInfo['dispname'] == 2) {
        $fromHandle = $fromUser['Anonymous'];
    } else {
        $fromHandle = $fromUser['handle'];
    }

    // Check if a reply and set the variables accordingly.
    if ($reply != 0) {
        $origMessage = GetMessageInfo($dbInfo, $reply);
        $toUserNo    = $origMessage['fromuserno'];
        $toUser      = $origMessage['fromhandle'];
        $toNetNode   = $origMessage['fromnetnode'];
        $toNetId     = $origMessage['fromnetid'];
    } else {
        $toUserNo    = 0;
        $toUser      = '';
        $toNetNode   = '';
        $toNetId     = 0;
    }
    $newString = PutLongInt($newString, $fromUserNo, 1);
    $newString = PutLongInt($newString, $toUserNo, 5);
    $newString = PutString($newString, $fromHandle, 9, 40);
    $newString = PutString($newString, $toUser, 49, 40);
    $newString = PutString($newString, $networks[1]['settings']['systemid'], 89, 80);
    $newString = PutString($newString, $toNetNode, 169, 80);
    $newString = PutInteger($newString, 1, 249);
    $newString = PutInteger($newString, $toNetId, 251);
    $newString = PutLongInt($newString, time() + (date("O") / 100 * 60 * 60), 253);
    $newString = PutLongInt($newString, WriteTotalMessageCount(), 257);
    $newString = PutString($newString, $msgSubject, 269, 64);
    $newString = PutInteger($newString, 0, 333);
    $newString = PutString($newString, date("D M d, Y H:i:s ") . $VA_MAIN['timezone'], 335, 38);
    $newString = PutInteger($newString, 0, 375);
    $newString = PutLongInt($newString, 0, 379);
    $newString = PutString($newString, '', 383, 12);
    $newString = PutString($newString, '', 395, 64);
    $newString = PutInteger($newString, 0, 463);
    $newString = PutString($newString, $dbInfo['filename'], 471, 8);
    $newString = PutLongInt($newString, $msgpos, 479);
    $newString = PutLongInt($newString, $msglen, 483);
    if ($reply != 0) $newString = PutLongInt($newString, $toNetId, 487);
    $newString = PutLongInt($newString, 0, 491);

    $fileString = ReadCfgFile($datFile);            // Read the file.
    $fileString = $fileString . $newString;         // Append the new data to the original file string.
    $status = WriteCfgFile($datFile, $fileString);  // Write the new data file.

    // Add the message to the .BIN file.
    $fp = fopen($binFile, 'a');                 // Open the file.
    flock($fp, 2);                              // Lock the file.
    $status = fwrite($fp, $msgBody . "\r\n");   // Write the message.
    flock($fp, 3);                              // Unlock the file.
    fclose($fp);                                // Close the file.

    // Create validation file or spool.net file based on db settings.
    if ($dbInfo['validate'] == 1) {
        $file = $VA_SYSPATHS['data'] . '/validate.0';   // Create local post validation file.
    } else {
        $file = $VA_SYSPATHS['data'] . '/spool.net';    // Create network spool file.
    }
    $fp = fopen($file, 'a');                                // Open the file.
    flock($fp, 2);                                          // Lock the file.
    $status = fwrite($fp, $newString . $msgBody . "\r\n");  // Write the message.
    flock($fp, 3);                                          // Unlock the file.
    fclose($fp);                                            // Close the file.

    return $status;                         // Return the write status.
}


/**
* Va-Database.inc.php :: WriteEmail()
*
* This function will write the email header informatoin and the message text
* to the proper files based on th passed database information. This function
* is to write only new email messages. For message area or files, use the
* appropriate function. If this is a reply to a message, set $reply to the
* message number of the original message.
*
* The toUserString is in the format of user@node*netnumber, for example:
* to send an email to me, it would be 1@1*80. Local messages are given a network
* number of 0.
*
*/
function WriteEmail($fromUserNo, $toUserString, $msgSubject, $msgBody, $reply=0, $attFile='', $oneliner=0) {
    if ((!is_int($fromUserNo))   && (!is_string($fromUserNo))) trigger_error(VA_Error(1));
    if ((!is_int($toUserString)) && (!is_string($toUserString))) trigger_error(VA_Error(1));
    if (!is_string($msgSubject)) trigger_error(VA_Error(1));
    if (!is_string($msgBody))    trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                    // Include global variable.
    global $CONFIG;                         // Include VADV-PHP Configuration.

    $fromUserNo   = trim($fromUserNo);
    if ($fromUserNo < 1) return FALSE;
    $toUserString = trim($toUserString);
    $msgSubject   = rtrim(strip_tags($msgSubject));
    if (strlen($msgSubject) > 64) $msgSubject = substr($msgSubject, 0, 64);
    $msgBody      = rtrim(wordwrap(strip_tags($msgBody), 80, "\r\n", 1));
    $file = $VA_SYSPATHS['text'] . '/email.tag';                            // Check for email tagline file.
    if (is_file($file)) $msgBody = $msgBody . "\r\n" . ReadCfgFile($file);  // Add the tagline if there.

    include_once('va-userfile.inc.php');
    include_once('va-main.inc.php');
    include_once('va-networks.inc.php');
    include_once('va-stats.inc.php');
    global $VA_MAIN;                        // Include global variable in case it is already defined.
    
    $networks = GetNetworkInfo();           // Get the local network info.
    if ($oneliner == 0) {
        $dbInfo = SetEmailInfo($fromUserNo);
    } else {
        $dbInfo = SetOnelinerInfo();
    }
    
    // Setup file variables if a file was attached.
    if ((trim($attFile) != '') && (is_file($attFile))) {
        $attFilesize = filesize($attFile);
        $pos = strrpos($attFile, '/');
        if ($pos == 0) $pos = strrpos($attFile, "\\");
        if ($pos > 0) {
            $attFilename = substr($attFile, $pos + 1);
            if ($CONFIG['hostname'] != '') {
                $attFilepath = ConvertUnc($attFile);
            } else {
                $attFilepath = $attFile;
            }
        } else {
            $attFilesize = 0;
            $attFilename = '';
            $attFilepath = '';
        }
    } else {
        $attFilesize = 0;
        $attFilename = '';
        $attFilepath = '';
    }

    // Break down the toUserString variable.
    $atpos = strpos($toUserString, '@');
    $aspos = strpos($toUserString, '*');
    $toUserNo  = trim(substr($toUserString, 0, $atpos));
    $toNetNode = trim(substr($toUserString, $atpos + 1, $aspos - $atpos - 1));
    $toNetId   = trim(substr($toUserString, $aspos + 1));

    // Check to see if it a local net id.
    if (($toNetId == 1) && ($toNetNode == $networks[1]['settings']['systemid'])) {
        $toNetNode = 0;
        $toNetId   = 0;
    }

    // If a local email, check to see if the user exists.
    if ($toNetId == 0) {
        if (is_numeric($toUserNo)) {
            $toUser = GetUser($toUserNo);
        } else {
            $toUser = FindHandle($toUserNo);
            if ($toUser == 0) {
                $toUser = FindRealName($toUserNo);
                if ($toUser == 0) die('<br /><h4>The user specified is invalid. Please press the BACK button on your browser.</h4>');
            }
            $toUserNo = $toUser;
            $toUser = GetUser($toUser);
        }
        // Check if the user has their email forwarded.
        if ($toUser['mailfwdflag'] == 1) {
            // If forwarded to another local user...
            if (($toUser['mailfwdnetno'] == 1) && ($toUser['mailfwdnode'] == $networks[1]['settings']['systemid'])) {
                $toUserNo = $toUser['mailfwduserno'];
                $toUser = GetUser($toUserNo);
            } else {
                // If forwarded to another location...
                if ($toUser['mailfwduserno'] == 0) {    // If the to user no field is blank...
                    $toUserNo = $toUser['mailfwdhandle'];
                } else {                                // If the to user no field is not blank...
                    $toUserNo = $toUser['mailfwduserno'];
                }
                $toNetNode = $toUser['mailfwdnode'];
                $toNetId   = $toUser['mailfwdnetno'];
            }
        }
    }

    // Set the toUserHandle variable.
    if ($toNetId == 0) {
        $toUserHandle = $toUser['handle'];
        $toNetNode    = $networks[1]['settings']['systemid'];
    } else {
        if (is_numeric($toUserNo)) {
            $toUserHandle = '';
        } else {
            $toUserHandle = $toUserNo;
            $toUserNo = 0;
        }
    }

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $binFile = $file . '.BIN';              // Set the bin filename.
    $datFile = $file . '.DAT';              // Set the database filename.

    // Get the 'from' user information and update it.
    $fileString = ReadUserfile();
    $fromUser = GetUser($fromUserNo);
    $fromUser['totalemail']++;
    $fileString = PutUser($fileString, $fromUserNo, $fromUser);
    $status = WriteUserfile($fileString);

    // Update the BBS stats.
    $fileString = ReadStats();
    $stats = GetStats();
    $stats['emailstotal']++;
    $stats['emailstoday']++;
    $fileString = PutStats($fileString, $stats);
    $status = WriteStats($fileString);

    if ($toNetId != 0) {
        $msgpos = 0;
    } else {
        if (!is_file($binFile)) {
            $msgpos = 1;
        } else {
            $msgpos = filesize($binFile) + 1;   // Get the message starting position.
        }
    }
    $msglen = strlen($msgBody) + 2;             // Get the message length.

    // Add spacing of 582 bytes of null chars.
    $newString = str_pad('', 582, chr(0));      // Create the new blank string.

    $newString = PutLongInt($newString, $fromUserNo, 1);
    $newString = PutLongInt($newString, $toUserNo, 5);
    $newString = PutString($newString, $fromUser['handle'], 9, 40);
    $newString = PutString($newString, $toUserHandle, 49, 40);
    $newString = PutString($newString, $networks[1]['settings']['systemid'], 89, 80);
    $newString = PutString($newString, $toNetNode, 169, 80);
    $newString = PutInteger($newString, 1, 249);
    if ($toNetId == 0) {
        $newString = PutInteger($newString, 1, 251);
    } else {
        $newString = PutInteger($newString, $toNetId, 251);
    }
    $newString = PutLongInt($newString, time() + (date("O") / 100 * 60 * 60), 253);
    $newString = PutLongInt($newString, WriteTotalMessageCount(), 257);
    $newString = PutString($newString, $msgSubject, 269, 64);
    $newString = PutInteger($newString, 0, 333);
    $newString = PutString($newString, date("D M d, Y H:i:s ") . $VA_MAIN['timezone'], 335, 38);
    $newString = PutInteger($newString, 0, 375);
    $newString = PutLongInt($newString, $attFilesize, 379);
    $newString = PutString($newString, $attFilename, 383, 12);
    $newString = PutString($newString, $attFilepath, 395, 64);
    $newString = PutInteger($newString, 0, 463);
    $newString = PutString($newString, $dbInfo['filename'], 471, 8);
    $newString = PutLongInt($newString, $msgpos, 479);
    $newString = PutLongInt($newString, $msglen, 483);
    $newString = PutLongInt($newString, $toNetId, 487);
    $newString = PutLongInt($newString, 0, 491);

    // If a local message, write it to the database, if not, write it to spool.net.
    if ($toNetId == 0) {
        // Update the .DAT file (and update dbseq.dat).
        $fileString = ReadCfgFile($datFile);            // Read the file.
        $fileString = $fileString . $newString;         // Append the new data to the original file string.
        $status = WriteCfgFile($datFile, $fileString);  // Write the new data file.

        // Add the message to the .BIN file.
        $fp = fopen($binFile, 'a');                 // Open the file.
        flock($fp, 2);                              // Lock the file.
        $status = fwrite($fp, $msgBody . "\r\n");   // Write the message.
        flock($fp, 3);                              // Unlock the file.
        fclose($fp);                                // Close the file.
        
        // Send a one liner to the user if online.
        $oneliner = $fromUser['handle'] . ' just sent you some email!';
        SendOneLiner($oneliner, $toUserNo);
    } else {
        // Create validation file or spool.net file based on db settings.
        $file = $VA_SYSPATHS['data'] . '/spool.net';            // Create network spool file.
        $fp = fopen($file, 'a');                                // Open the file.
        flock($fp, 2);                                          // Lock the file.
        $status = fwrite($fp, $newString . $msgBody . "\r\n");  // Write the message.
        flock($fp, 3);                                          // Unlock the file.
        fclose($fp);                                            // Close the file.
    }

    return $status;                         // Return the write status.
}
    


/**
* Va-Database.inc.php :: QuoteMessage()
*
* This function will take the passed message and add > to the beginning of each
* line and truncate each line to 78 characters. This is used when quoting a
* message a user is replying to.
*
*/
function QuoteMessage($message) {
    $message = preg_replace("/\r\n|\n|\r/", "\r\n", StripHeartColors(strip_tags($message)));
    $message = explode("\r\n", $message);
    for ($x = 0; $x < count($message); $x++) {
        if (strlen($message[$x]) >= 78) $message[$x] = substr($message[$x], 0, 78);
    }
    $message = "\r\n\r\n> " . implode("\r\n> ", $message) . "\r\n";
    
    return $message;
}


/**
* Va-Database.inc.php :: GetMessageCount()
*
* This function will determine the number of messages contained within the
* specified database.
*
*/
function GetMessageCount($dbInfo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                    // Include the global variable.
    
    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];

    $datFile = $file . '.DAT';              // Add the datafile extension.
    if (!is_file($datFile)) return FALSE;

    $fileSize = filesize($datFile);         // Determine the size of the file.
    $msgNo    = intval($fileSize / 582);    // Determine the number of messages.

    return $msgNo;                          // Return the message count.
}


/**
* Va-Database.inc.php :: GetEmailCount()
*
* This function will determine the number of messages contained within the
* email database for the specified user. It returns an array containing
* the number of messages total (msgno) and the number of messages that have not
* been read (unread).
*
*/
function GetEmailCount($userNo) {
    if (!is_int($userNo) && (!is_string($userNo))) trigger_error(VA_Error(1));

    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $datFile    = $VA_SYSPATHS['db'] . '/email.dat';
    if (!is_file($datFile)) return FALSE;

    $fileString = ReadCfgFile($datFile);    // Read the file.

    $count['msgno']  = 0;                   // Set the default value.
    $count['unread'] = 0;
    $offset = 582;
    for ($x = 0; $offset < filesize($datFile); $x++) {
        $offset = 582 * $x;
        $user = GetLongInt($fileString, 5 + $offset);
        if ($user == $userNo) {
            $readFlag = GetInteger($fileString, 375 + $offset);
            if ($readFlag == 0) $count['unread']++;
            $count['msgno']++;
        }
    }

    return $count;                          // Return the message count.
}


/**
* Va-Database.inc.php :: GetMessagesTo()
*
* This function will scan the current database for all messages to the current
* user and store them into an array.
*
*/
function SearchMessagesTo($dbInfo, $userNo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($userNo))  && (!is_string($userNo))) trigger_error(VA_Error(1));
    if ($userNo == 0) {
        $msgTo[1] = '';
        return $msgTo;
    }

    include_once('va-networks.inc.php');    // Include network information.

    // This function has been optimized for speed and does not use the pre-
    // written functions above.
    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];

    $datFile = $file . '.DAT';              // Set the database filename.
    if (!is_file($datFile)) return FALSE;

    $netInfo    = GetNetworkInfo();
    $fileString = ReadCfgFile($datFile);    // Read the database file.

    $cnt     = 1;                             // Counter.
    $msgTo   = array();

    for ($msgNo = 1; $msgNo <= $dbInfo['msgno']; $msgNo++) {
        $offset = 582 * ($msgNo - 1);
        $msgInfo['touserno']  = GetLongInt($fileString, 5 + $offset);
        if ($msgInfo['touserno'] == $userNo) {
            $msgInfo['tonetid'] = GetInteger($fileString, 251 + $offset);
            if (isset($netInfo[$msgInfo['tonetid']]['netno'])) {
                if ($msgInfo['tonetid'] == $netInfo[$msgInfo['tonetid']]['netno']) {
                    $msgInfo['tonetnode'] = GetString($fileString, 169 + $offset, 80);
                    if ($msgInfo['tonetnode'] == $netInfo[$msgInfo['tonetid']]['settings']['systemid']) {
                        $msgTo[$cnt] = $msgNo;      // Add the message number to the array.
                        $cnt++;                     // Increase the counter by one.
                    }
                }
            }
        }
    }

    $msgTo[0] = $cnt - 1;                 // Set the total number of messages.

    return $msgTo;                        // Return the messages from this user.
}


/**
* Va-Database.inc.php :: GetMessagesFrom()
*
* This function will scan the current database for all messages from the
* current user and store them into an array.
*
*/
function SearchMessagesFrom($dbInfo, $userNo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    if ((!is_int($userNo))  && (!is_string($userNo))) trigger_error(VA_Error(1));
    if ($userNo == 0) {
        $msgFrom[1] = '';
        return $msgFrom;
    }
    
    include_once('va-networks.inc.php');    // Include network information.
    
    // This function has been optimized for speed and does not use the pre-
    // written functions above.
    global $VA_SYSPATHS;                    // Include global variable.

    // Set the filename.
    $file    = $VA_SYSPATHS['db'] . '/' . $dbInfo['filename'];
    $datFile = $file . '.DAT';              // Set the database filename.

    if (!is_file($datFile)) return FALSE;

    $netInfo    = GetNetworkInfo();
    $fileString = ReadCfgFile($datFile);    // Read the database file.

    $cnt     = 1;                             // Counter.
    $msgFrom = array();
    
    for ($msgNo = 1; $msgNo <= $dbInfo['msgno']; $msgNo++) {
        $offset = 582 * ($msgNo - 1);
        $msgInfo['fromuserno']  = GetLongInt($fileString, 1 + $offset);
        if ($msgInfo['fromuserno'] == $userNo) {
            $msgInfo['fromnetid'] = GetInteger($fileString, 249 + $offset);
            if (isset($netInfo[$msgInfo['fromnetid']]['netno'])) {
                if ($msgInfo['fromnetid'] == $netInfo[$msgInfo['fromnetid']]['netno']) {
                    $msgInfo['fromnetnode'] = GetString($fileString, 89 + $offset, 80);
                    if ($msgInfo['fromnetnode'] == $netInfo[$msgInfo['fromnetid']]['settings']['systemid']) {
                        $msgFrom[$cnt] = $msgNo;    // Add the message number to the array.
                        $cnt++;                     // Increase the counter by one.
                    }
                }
            }
        }
    }
    
    $msgFrom[0] = $cnt - 1;                 // Set the total number of messages.
    
    return $msgFrom;                        // Return the messages from this user.
}


/**
* Va-Database.inc.php :: SetOnelinerInfo()
*
* This function will return the oneliner database information in the same format
* as dbInfo.
*
*/
function SetOnelinerInfo() {
    global $VA_SYSPATHS;
    global $CONFIG;
    include_once('va-main.inc.php');
    global $VA_MAIN;

    $onelinerInfo['topicno']    = '';
    $onelinerInfo['topic']      = 'Oneliners';
    $onelinerInfo['dbno']       = 0;
    $onelinerInfo['filename']   = 'ONELINE';
    $onelinerInfo['filepath']   = '';
    $onelinerInfo['desc']       = 'Oneliners';
    $onelinerInfo['private']    = 0;
    $onelinerInfo['dbgroup']    = '';
    $onelinerInfo['maxmsg']     = 0;
    $onelinerInfo['minreadsl']  = 0;
    $onelinerInfo['minwritesl'] = 0;
    $onelinerInfo['agelimit']   = 0;
    $onelinerInfo['accflags']   = 0;
    $onelinerInfo['tagline']    = 0;
    $onelinerInfo['dispname']   = 0;
    $onelinerInfo['randtitle']  = 0;
    $onelinerInfo['qwkconfno']  = 0;
    $onelinerInfo['moderator']  = 0;
    $onelinerInfo['validate']   = 0;
    $onelinerInfo['freqable']   = 0;
    $onelinerInfo['devicetype'] = 0;
    $onelinerInfo['dbtype']     = 'E';
    $onelinerInfo['msgno']      = GetMessageCount($onelinerInfo);

    return $onelinerInfo;                      // Return the oneliner database information.
}


/**
* Va-Database.inc.php :: SetEmailInfo()
*
* This function will return the email database information in the same format
* as dbInfo.
*
*/
function SetEmailInfo($userno) {
    global $VA_SYSPATHS;
    global $CONFIG;
    include_once('va-main.inc.php');
    global $VA_MAIN;
    
    $emailInfo['topicno']    = '';
    $emailInfo['topic']      = 'Email';
    $emailInfo['dbno']       = 0;
    $emailInfo['filename']   = 'EMAIL';
    if ($CONFIG['filehost']) {
        $emailInfo['filepath'] = ConvertUnc($VA_SYSPATHS['sysop']);
    } else {
        $emailInfo['filepath'] = $VA_SYSPATHS['sysop'];
    }
    $emailInfo['desc']       = 'Personal Email';
    $emailInfo['private']    = 0;
    $emailInfo['dbgroup']    = '';
    $emailInfo['maxmsg']     = 0;
    $emailInfo['minreadsl']  = 0;
    $emailInfo['minwritesl'] = $VA_MAIN['emlothers'];
    $emailInfo['agelimit']   = 0;
    $emailInfo['accflags']   = 0;
    $emailInfo['tagline']    = 0;
    $emailInfo['dispname']   = 0;
    $emailInfo['randtitle']  = 0;
    $emailInfo['qwkconfno']  = 0;
    $emailInfo['moderator']  = 0;
    $emailInfo['validate']   = 0;
    $emailInfo['freqable']   = 0;
    $emailInfo['devicetype'] = 0;
    $emailInfo['dbtype']     = 'E';
    $emailInfo['msgno']      = GetMessageCount($emailInfo);
    $emailInfo['msgcnt']     = GetEmailCount($userno);

    return $emailInfo;                      // Return the email database information.
}


/**
* Va-Database.inc.php :: GetLastPostInfo()
*
* This function will get the last post's message information of the
* specified database.
*
*/
function GetLastPostInfo($dbInfo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    
    $lastPost = GetMessageInfo($dbInfo, $dbInfo['msgno']);

    return $lastPost;                       // Return the lastpost information.
}


/**
* Va-Database.inc.php :: DbAccess()
*
* This function will determine if the user has access clearance for the specified
* database.
*
*/
function DbAccess($dbInfo) {
    if (!is_array($dbInfo)) trigger_error(VA_Error(1));
    
    global $USER;                           // Include the global variable.
    $access = FALSE;                        // Set the default access value, false.
    
    // If the sysop, give access regardless.
    if ($USER['securitylevel'] == 255) return TRUE;
    
    // Check security level, age and the user access flags.
    if (($USER['securitylevel'] >= $dbInfo['minreadsl']) &&
        ($USER['age'] >= $dbInfo['agelimit']) &&
        (($dbInfo['accflags'] == '') || (strpos($USER['accflags'], $dbInfo['accflags']) !== FALSE)))
        $access = TRUE;

    return $access;                         // Return the access result.
}

?>
