<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Userfile.inc.php                                               04/23/2003
 * This script will read the userfile.dat file for the specified user and store
 * its information into an array.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2003 Steve Winn. All Rights Reserved.
 *
 * The source code contains a fair amount of comments, which may aid anyone
 * attempting to follow the program or to modify it.
 *
 * For licensing and copyright information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('va-functions.inc.php');       // A VADV-PHP script.


/**
* Va-Userfile.inc.php :: GetUser()
*
* This function will retrieve the information stored in the userfile.dat file
* for the specified user number. This file contains all of the user's data.
*
*/
function GetUser($userNo) {
    if ((!is_int($userNo)) && (!is_string($userNo))) trigger_error(VA_Error(1));
    if ($userNo <= 0) return 0;
    
    $fileString = ReadUserFile();
    if ($fileString == FALSE) trigger_error(VA_Error(3));

    $user   = array();
    $offset = 1048 * ($userNo - 1);
    $user['userno']        = $userNo;
    $user['handle']        = GetString($fileString, 1 + $offset, 40);
    $user['realname']      = GetString($fileString, 41 + $offset, 40);
    $user['address']       = GetString($fileString, 81 + $offset, 40);
    $user['city']          = GetString($fileString, 121 + $offset, 32);
    $user['state']         = GetString($fileString, 153 + $offset, 3);
    $user['zipcode']       = GetString($fileString, 156 + $offset, 10);
    $user['phone1']        = GetString($fileString, 166 + $offset, 12);
    $user['phone2']        = GetString($fileString, 178 + $offset, 12);
    // User password is encrypted with md5 for security purposes.
    $user['password']      = md5(strtoupper(GetString($fileString, 190 + $offset, 8)));
    $user['mailfwdflag']   = GetInteger($fileString, 198 + $offset);
    $user['mailfwdhandle'] = GetString($fileString, 200 + $offset, 40);
    $user['mailfwduserno'] = GetLongInt($fileString, 240 + $offset);
    $user['mailfwdnode']   = GetString($fileString, 244 + $offset, 80);
    $user['mailfwdnetno']  = GetInteger($fileString, 324 + $offset);
//    $user['lngdistcost']   = GetSingle($fileString, 326 + $offset);
//    $user['lngdistacc']    = GetSingle($fileString, 330 + $offset);
    $user['sortfilelst']   = GetInteger($fileString, 334 + $offset);
    $user['pagebreak']     = GetInteger($fileString, 336 + $offset);
    $user['bdaymonth']     = GetInteger($fileString, 338 + $offset);
    $user['bdayday']       = GetInteger($fileString, 340 + $offset);
    $user['bdayyear']      = GetInteger($fileString, 342 + $offset);
    $user['age']           = CalcAge($user['bdaymonth'], $user['bdayday'], $user['bdayyear']);
    $user['flags']         = str_replace(' ', '', GetString($fileString, 344 + $offset, 26));
    $user['accflags']      = str_replace(' ', '', GetAccFlags(GetLongInt($fileString, 370 + $offset)));
    $user['securitylevel'] = GetInteger($fileString, 374 + $offset);
    $user['laston']        = GetString($fileString, 376 + $offset, 6);
    $user['maxtime']       = GetInteger($fileString, 382 + $offset);
    $user['timecounter']   = GetInteger($fileString, 384 + $offset);
    $user['totalcalls']    = GetLongInt($fileString, 386 + $offset);
    $user['totaltime']     = GetLongInt($fileString, 390 + $offset);
    $user['videomode']     = GetInteger($fileString, 394 + $offset);
    $user['ratioexempt']   = GetInteger($fileString, 396 + $offset);
    $user['verifybday']    = GetInteger($fileString, 398 + $offset);
    $user['textcolor']     = GetChar($fileString, 400 + $offset);
    $user['basecolor']     = GetChar($fileString, 401 + $offset);
    $user['promptcolor']   = GetChar($fileString, 402 + $offset, 1);
    $user['inputcolor']    = GetChar($fileString, 403 + $offset, 1);
    $user['boxcolor']      = GetChar($fileString, 404 + $offset, 1);
    $user['mnuselcolor']   = GetChar($fileString, 405 + $offset, 1);
    $user['headercolor']   = GetChar($fileString, 406 + $offset, 1);
    $user['extra1']        = GetString($fileString, 407 + $offset, 12);
    $user['extra2']        = GetString($fileString, 419 + $offset, 12);
    $user['extra3']        = GetString($fileString, 431 + $offset, 12);
    $user['extra4']        = GetString($fileString, 443 + $offset, 12);
    $user['extra5']        = GetString($fileString, 455 + $offset, 12);
    $user['extra6']        = GetString($fileString, 467 + $offset, 12);
    $user['extra7']        = GetString($fileString, 479 + $offset, 12);
    $user['extra8']        = GetString($fileString, 491 + $offset, 12);
    $user['macro1']        = GetString($fileString, 503 + $offset, 160);
    $user['macro2']        = GetString($fileString, 663 + $offset, 160);
    $user['uplfiles']      = GetLongInt($fileString, 823 + $offset);
    $user['uplkb']         = GetLongInt($fileString, 827 + $offset);
    $user['dlfiles']       = GetLongInt($fileString, 831 + $offset);
    $user['dlkb']          = GetLongInt($fileString, 835 + $offset);
    $user['totalposts']    = GetLongInt($fileString, 839 + $offset);
    $user['totalemail']    = GetLongInt($fileString, 843 + $offset);
    $user['postsread']     = GetLongInt($fileString, 847 + $offset);
    $user['timebank']      = GetInteger($fileString, 851 + $offset);
    $user['credits']       = GetLongInt($fileString, 853 + $offset);
    $user['comment']       = GetString($fileString, 857 + $offset, 16);
    $user['protocol']      = GetString($fileString, 873 + $offset, 1);

    return $user;                           // Return the user information.
}


/**
* Va-Userfile.inc.php :: PutUser()
*
* This function will write updated information to the specified user's account.
*
*/
function PutUser($fileString, $userNo, $configVariable) {
    if (!is_string($fileString))    trigger_error(VA_Error(1));
    if ((!is_int($userNo))          && (!is_string($userNo))) trigger_error(VA_Error(1));
    if (!is_array($configVariable)) trigger_error(VA_Error(1));
    
    $offset = 1048 * ($userNo - 1);
    if (isset($configVariable['handle']))        $fileString = PutString($fileString, $configVariable['handle'], 1 + $offset, 40);
    if (isset($configVariable['realname']))      $fileString = PutString($fileString, $configVariable['realname'], 41 + $offset, 40);
    if (isset($configVariable['address']))       $fileString = PutString($fileString, $configVariable['address'], 81 + $offset, 40);
    if (isset($configVariable['city']))          $fileString = PutString($fileString, $configVariable['city'], 121 + $offset, 32);
    if (isset($configVariable['state']))         $fileString = PutString($fileString, $configVariable['state'], 153 + $offset, 3);
    if (isset($configVariable['zipcode']))       $fileString = PutString($fileString, $configVariable['zipcode'], 156 + $offset, 10);
    if (isset($configVariable['phone1']))        $fileString = PutString($fileString, $configVariable['phone1'], 166 + $offset, 12);
    if (isset($configVariable['phone2']))        $fileString = PutString($fileString, $configVariable['phone2'], 178 + $offset, 12);
    // To change the passwords, they must be in plain text and less than 8 chars.
    // If it is encrypted with md5, then it has a length of 32 characters and will not write it.
    if ((isset($configVariable['password'])) &&
        (strlen($configVariable['password']) <= 8))
                                                 $fileString = PutString($fileString, $configVariable['password'], 190 + $offset, 8);
    if (isset($configVariable['mailfwdflag']))   $fileString = PutInteger($fileString, $configVariable['mailfwdflag'], 198 + $offset);
    if (isset($configVariable['mailfwdhandle'])) $fileString = PutString($fileString, $configVariable['mailfwdhandle'], 200 + $offset, 40);
    if (isset($configVariable['mailfwduserno'])) $fileString = PutLongInt($fileString, $configVariable['mailfwduserno'], 240 + $offset);
    if (isset($configVariable['mailfwdnode']))   $fileString = PutString($fileString, $configVariable['mailfwdnode'], 244 + $offset, 80);
    if (isset($configVariable['mailfwdnetno']))  $fileString = PutInteger($fileString, $configVariable['mailfwdnetno'], 324 + $offset);
//    if (isset($configVariable['lngdistcost']))   $fileString = PutSingle($fileString, $configVariable['lngdistcost'], 326 + $offset);
//    if (isset($configVariable['lngdistacc']))    $fileString = PutSingle($fileString, $configVariable['lngdistacc'], 330 + $offset);
    if (isset($configVariable['sortfilelst']))   $fileString = PutInteger($fileString, $configVariable['sortfilelst'], 334 + $offset);
    if (isset($configVariable['pagebreak']))     $fileString = PutInteger($fileString, $configVariable['pagebreak'], 336 + $offset);
    if (isset($configVariable['bdaymonth']))     $fileString = PutInteger($fileString, $configVariable['bdaymonth'], 338 + $offset);
    if (isset($configVariable['bdayday']))       $fileString = PutInteger($fileString, $configVariable['bdayday'], 340 + $offset);
    if (isset($configVariable['bdayyear']))      $fileString = PutInteger($fileString, $configVariable['bdayyear'], 342 + $offset);
    if (isset($configVariable['flags']))         $fileString = PutString($fileString, MakeFlags($configVariable['flags']), 344 + $offset, 26);
    if (isset($configVariable['accflags']))      $fileString = PutLongInt($fileString, MakeAccFlags($configVariable['accflags']), 370 + $offset);
    if (isset($configVariable['securitylevel'])) $fileString = PutInteger($fileString, $configVariable['securitylevel'], 374 + $offset);
    if (isset($configVariable['laston']))        $fileString = PutString($fileString, $configVariable['laston'], 376 + $offset, 6);
    if (isset($configVariable['maxtime']))       $fileString = PutInteger($fileString, $configVariable['maxtime'], 382 + $offset);
    if (isset($configVariable['timecounter']))   $fileString = PutInteger($fileString, $configVariable['timecounter'], 384 + $offset);
    if (isset($configVariable['totalcalls']))    $fileString = PutLongInt($fileString, $configVariable['totalcalls'], 386 + $offset);
    if (isset($configVariable['totaltime']))     $fileString = PutLongInt($fileString, $configVariable['totaltime'], 390 + $offset);
    if (isset($configVariable['videomode']))     $fileString = PutInteger($fileString, $configVariable['videomode'], 394 + $offset);
    if (isset($configVariable['ratioexempt']))   $fileString = PutInteger($fileString, $configVariable['ratioexempt'], 396 + $offset);
    if (isset($configVariable['verifybday']))    $fileString = PutInteger($fileString, $configVariable['verifybday'], 398 + $offset);
    if (isset($configVariable['textcolor']))     $fileString = PutChar($fileString, $configVariable['textcolor'], 400 + $offset);
    if (isset($configVariable['basecolor']))     $fileString = PutChar($fileString, $configVariable['basecolor'], 401 + $offset);
    if (isset($configVariable['promptcolor']))   $fileString = PutChar($fileString, $configVariable['promptcolor'], 402 + $offset, 1);
    if (isset($configVariable['inputcolor']))    $fileString = PutChar($fileString, $configVariable['inputcolor'], 403 + $offset, 1);
    if (isset($configVariable['boxcolor']))      $fileString = PutChar($fileString, $configVariable['boxcolor'], 404 + $offset, 1);
    if (isset($configVariable['mnuselcolor']))   $fileString = PutChar($fileString, $configVariable['mnuselcolor'], 405 + $offset, 1);
    if (isset($configVariable['headercolor']))   $fileString = PutChar($fileString, $configVariable['headercolor'], 406 + $offset, 1);
    if (isset($configVariable['extra1']))        $fileString = PutString($fileString, $configVariable['extra1'], 407 + $offset, 12);
    if (isset($configVariable['extra2']))        $fileString = PutString($fileString, $configVariable['extra2'], 419 + $offset, 12);
    if (isset($configVariable['extra3']))        $fileString = PutString($fileString, $configVariable['extra3'], 431 + $offset, 12);
    if (isset($configVariable['extra4']))        $fileString = PutString($fileString, $configVariable['extra4'], 443 + $offset, 12);
    if (isset($configVariable['extra5']))        $fileString = PutString($fileString, $configVariable['extra5'], 455 + $offset, 12);
    if (isset($configVariable['extra6']))        $fileString = PutString($fileString, $configVariable['extra6'], 467 + $offset, 12);
    if (isset($configVariable['extra7']))        $fileString = PutString($fileString, $configVariable['extra7'], 479 + $offset, 12);
    if (isset($configVariable['extra8']))        $fileString = PutString($fileString, $configVariable['extra8'], 491 + $offset, 12);
    if (isset($configVariable['macro1']))        $fileString = PutString($fileString, $configVariable['macro1'], 503 + $offset, 160);
    if (isset($configVariable['macro2']))        $fileString = PutString($fileString, $configVariable['macro2'], 663 + $offset, 160);
    if (isset($configVariable['uplfiles']))      $fileString = PutLongInt($fileString, $configVariable['uplfiles'], 823 + $offset);
    if (isset($configVariable['uplkb']))         $fileString = PutLongInt($fileString, $configVariable['uplkb'], 827 + $offset);
    if (isset($configVariable['dlfiles']))       $fileString = PutLongInt($fileString, $configVariable['dlfiles'], 831 + $offset);
    if (isset($configVariable['dlkb']))          $fileString = PutLongInt($fileString, $configVariable['dlkb'], 835 + $offset);
    if (isset($configVariable['totalposts']))    $fileString = PutLongInt($fileString, $configVariable['totalposts'], 839 + $offset);
    if (isset($configVariable['totalemail']))    $fileString = PutLongInt($fileString, $configVariable['totalemail'], 843 + $offset);
    if (isset($configVariable['postsread']))     $fileString = PutLongInt($fileString, $configVariable['postsread'], 847 + $offset);
    if (isset($configVariable['timebank']))      $fileString = PutInteger($fileString, $configVariable['timebank'], 851 + $offset);
    if (isset($configVariable['credits']))       $fileString = PutLongInt($fileString, $configVariable['credits'], 853 + $offset);
    if (isset($configVariable['comment']))       $fileString = PutString($fileString, $configVariable['comment'], 857 + $offset, 16);
    if (isset($configVariable['protocol']))      $fileString = PutString($fileString, $configVariable['protocol'], 873 + $offset, 1);

    return $fileString;                     // Return the new file string.
}


/**
* Va-Userfile.inc.php :: ReadUserfile()
*
* This function will read the userfile.dat file.
*
*/
function ReadUserfile() {
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['data'] . '/userfile.dat';
    if (!is_file($file)) return FALSE;

    $fileString = ReadCfgFile($file);       // Read the file.

    return $fileString;                     // Return the file string.
}


/**
* Va-Userfile.inc.php :: WriteUserfile()
*
* This function will write a new userfile.dat file based on the file string
* passed to it. If the file string is not in the correct format, then your
* userfile.dat file will become corrupt and the BBS could stop functioning!
*
*/
function WriteUserfile($fileString) {
    if (!is_string($fileString)) trigger_error(VA_Error(1));
    
    global $VA_SYSPATHS;                    // Include the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['data'] . '/userfile.dat';
    if (!is_file($file)) return FALSE;

    $status = WriteCfgFile($file, $fileString);

    return $status;                         // Return the result status.
}


/**
* Va-Userfile.inc.php :: FindHandle()
*
* This function will search the userfile.dat file for the $handle and return
* the user number.
*
*/
function FindHandle($handle) {
    // Check to see if the variable was passed, if not then return zero.
    if ($handle == '') return 0;

    global $VA_SYSPATHS;                    // Pass the global variable.

    // Set the filename.
    $file = $VA_SYSPATHS['data'] . '/userfile.dat';
    if (!is_file($file)) return FALSE;
    
    $fileString = ReadCfgFile($file);       // Read the file.

    $length = strlen($fileString);          // Determine the length of the file string.

    $handle = strtoupper($handle);          // Change the handle to uppercase.

    $offset   = 1048;                       // Set the file offset.
    $position = 1;                          // Set the starting position.
    $num      = 0;                          // Set the starting user number.
    $result   = 1;                          // Set the default result (non-zero).
    $seclvl   = 0;

    for ($x = 1; $result != 0; $x++) {
        $string   = GetString($fileString, $position, 40);
        $seclvl   = GetInteger($fileString, $position + 373);
        $string   = strtoupper($string);        // Change the string to uppercase.
        $result   = strcmp($handle, $string);   // Compare the handle to the string.
        $position = $position + $offset;        // Goto the next user record.
        $num++;                                 // Increase the user number.
        if ($result == 0) {                     // If they match...
           if ($seclvl == 0) $result = 1;       // If the user is deleted, reset the result back to 1.
        }
        if ($position > $length) {              // If you reach the end of the file...
            $result = 0;                        // Set the result to end the loop...
            $num    = 0;                        // And the user number is now zero.
        }
    }
    
    if ($seclvl == 0) return 0;             // If the user is deleted, return 0.
    
    return $num;                            // Return the user number.
}


/**
* Va-Userfile.inc.php :: FindRealName()
*
* This function will search the userfile.dat file for the real $name and return
* the user number.
*
*/
function FindRealName($name) {
    // Check to see if the variable was passed, if not then return zero.
    if ($name == '') return 0;

    global $VA_SYSPATHS;                    // Pass the global variable.

    // Set the filename.
    $file       = $VA_SYSPATHS['data'] . '/userfile.dat';
    $fileString = ReadCfgFile($file);       // Read the file.
    $length     = strlen($fileString);      // Determine the length of the file string.

    $name       = strtoupper($name);        // Change the name to uppercase.

    $offset     = 1048;                     // Set the file offset.
    $position   = 41;                       // Set the starting position.
    $num        = 0;                        // Set the starting user number.
    $result     = 1;                        // Set the default result (non-zero).
    $seclvl     = 0;

    for ($x = 1; $result != 0; $x++) {
        $string   = GetString($fileString, $position, 40);
        $seclvl   = GetInteger($fileString, 333 + $position);
        $string   = strtoupper($string);    // Change the string to uppercase.
        $result   = strcmp($name, $string); // Compare the name to the string.
        $position = $position + $offset;    // Goto the next user record.
        $num++;                             // Increase the user number.
        if ($result == 0) {                 // If they match...
           if ($seclvl == 0) $result = 1;   // If the user is deleted, reset the result back to 1.
        }
        if ($position > $length) {          // If you reach the end of the file...
            $result = 0;                    // Set the result to end the loop...
            $num    = 0;                    // And the user number is now zero.
        }
    }
    
    if ($seclvl == 0) return 0;             // If the user is deleted, return 0.

    return $num;                            // Return the user number.
}


/**
* Va-Userfile.inc.php :: CalcDayNumber()
*
* This function will determine the day number of the specified date.
*
*/
function CalcDayNumber($month, $day, $year) {
    $month = ($month + 9) % 12;
    $year  = $year - $month/10;

    return 365*$year + $year/4 - $year/100 + $year/400 + ($month*306 + 5)/10 + $day - 1;
}


/**
* Va-Userfile.inc.php :: CalcAge()
*
* This function will calculate the age in years of the passed month, day and
* year date.
*
*/
function CalcAge($month, $day, $year) {
    $today = CalcDayNumber(date('m'), date('d'), date('Y'));
    $date  = CalcDayNumber($month, $day, $year);

    return intval(($today - $date) / 365);
}
?>
